#include "board.h"
#include "rom_default.h"

//speed factor = hfreq / 33716Hz
//cycles = 1214 / hfreq[khz];

uint8_t acc;
uint8_t rx;
uint8_t ry;
uint8_t io_out;
uint16_t pc;

typedef struct { // TTL state that the CPU controls
	uint8_t IR, D;
} CpuState;

uint8_t __attribute__((section(".simrom"))) rom_data[65536][2];
uint8_t  sim_ram[32768]; 
volatile uint8_t io_in;

CpuState cpuCycle(const CpuState S)
{
	CpuState T = S; // New state is old state unless something changes

	T.IR = rom_data[pc][0]; // Instruction Fetch
	T.D  = rom_data[pc][1];

	int ins = S.IR >> 5;       // Instruction
	int mod = (S.IR >> 2) & 7; // Addressing mode (or condition)
	int bus = S.IR&3;          // Busmode
	int W = (ins == 6);        // Write instruction?
	int J = (ins == 7);        // Jump instruction?

	uint8_t lo=S.D, hi=0, *to=NULL; // Mode Decoder
	int incX=0;
	if (!J)
		switch (mod) {
			#define E(p) (W?0:p) // Disable AC and OUT loading during sim_ram write
			case 0: to=E(&acc);                          break;
			case 1: to=E(&acc); lo=rx;                  break;
			case 2: to=E(&acc);         hi=ry;          break;
			case 3: to=E(&acc); lo=rx; hi=ry;          break;
			case 4: to=  &rx;                            break;
			case 5: to=  &ry;                            break;
			case 6: to=E(&io_out);                         break;
			case 7: to=E(&io_out); lo=rx; hi=ry; incX=1; break;
		}

	uint16_t addr = (hi << 8) | lo;

	int B = 0xff; // Data Bus
	switch (bus) {
		case 0: B=S.D;                        break;
		case 1: if (!W) B = sim_ram[addr&0x7fff]; break;
		case 2: B=acc;                       break;
		case 3: B=io_in;                         break;
	}

	if (W) sim_ram[addr&0x7fff] = B; // Random Access Memory

	uint8_t ALU; // Arithmetic and Logic Unit

	switch (ins) {
		case 0: ALU =        B; break; // LD
		case 1: ALU = acc & B; break; // ANDA
		case 2: ALU = acc | B; break; // ORA
		case 3: ALU = acc ^ B; break; // XORA
		case 4: ALU = acc + B; break; // ADDA
		case 5: ALU = acc - B; break; // SUBA
		case 6: ALU = acc;     break; // ST
		case 7: ALU = -acc;    break; // Bcc/JMP
	}

	if (to) *to = ALU; // Load value into register
	if (incX) rx = rx + 1; // Increment X

	pc = pc + 1; // Next instruction
	if (J) {
		if (mod != 0) { // Conditional branch within page
			int cond = (acc>>7) + 2*(acc==0);
			if (mod & (1 << cond)) // 74153
				pc = (pc & 0xff00) | B;
		} else
			pc = (ry << 8) | B; // Unconditional far jump
	}
	return T;
}

int main(void)
{
	CpuState S,T;
	long i;
	int c=0;

	unilib_init();
	set_clock(CLOCK_16_168);
	
	set_port_output(PORT_A);
	set_portpin_output(PORT_B,7);
	set_portpin_output(PORT_B,5);
	set_portpin_one(PORT_B,7);
	
	//copy the rom_data image
	for(i=0;i<(65536);i++)
	{
		 rom_data[i][0] = rom_default[2*i];
		 rom_data[i][1] = rom_default[2*i+1];
	}
	
	pc=0;
	T=cpuCycle(S); // Update CPU
	S=T;
	pc=0;
		
	while(1)
	{
		T = cpuCycle(S); 		// Update CPU
		GPIOA.ODR.REG=io_out & 0xff;	//video out
//		set_portpin_level(PORT_B,5,c);
//		c=1-c;
		S=T;
	}
	
	return 0;
}